(***************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                           BIALYSTOK 2018                               **
**                                                                        **
**                          www serwer config                             **
**                      Wersja z dnia : 12.19.2018                        **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
***************************************************************************)

unit SerwCfg_unit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, ueled, SdpoSerial, Forms, Controls, Graphics,
  Dialogs, StdCtrls, ExtCtrls, synaser;


const
  CyclicBufferSize             = 256 ;

type
  CyclicBufferRecT             = record
                                   FrIndex        : word ;
                                   ToIndex        : word ;
                                   Buffer         : array [ 0 .. CyclicBufferSize - 1 ] of char ;
                                 end (* record *) ;

  { TSerwerCfgForm }

  TSerwerCfgForm = class ( TForm )
                     ResetButton         : TButton ;
                     SendButton          : TButton ;
                     OpenPortButton      : TButton ;
                     ExitButton          : TButton ;
                     ClosePortButton     : TButton ;
                     CfgDataLabel        : TLabel ;
                     PortLabel           : TLabel ;
                     SerialPortComboBox  : TComboBox ;
                     CfgDataEdit         : TEdit ;
                     TextMemo            : TMemo ;
                     SerialChannel       : TSdpoSerial ;
                     ConnectLED          : TuELED ;
                     GeneralTimer        : TTimer ;
                     procedure FormCreate ( Sender : TObject ) ;
                     procedure OpenPortButtonClick ( Sender : TObject ) ;
                     procedure ClosePortButtonClick(Sender: TObject);
                     procedure ExitButtonClick ( Sender : TObject ) ;
                     procedure ResetButtonClick(Sender: TObject);
                     procedure SendButtonClick(Sender: TObject);
                     procedure GeneralTimerTimer ( Sender : TObject ) ;
                     procedure EnableTotalButtons ;
                     procedure DisableTotalButtons ;
                     procedure AddToCyclic ( Ch : char ) ;
                     function GetCh ( ) : char ;
  private
    { private declarations }
  public
    { public declarations }
                     CyclicBuffer        : CyclicBufferRecT ;
                     RecvCommand         : string ;
  end;

var
  SerwerCfgForm: TSerwerCfgForm;

implementation

{$R *.lfm}

{ TSerwerCfgForm }

procedure TSerwerCfgForm.FormCreate ( Sender : TObject ) ;
begin (* TSerwerCfgForm.FormCreate *)
  SerialPortComboBox . Items . CommaText := GetSerialPortNames ;
  SerialPortComboBox . Text := SerialPortComboBox . Items [ 0 ] ;
  GeneralTimer . Enabled := false ;
  DisableTotalButtons ;
end (* TSerwerCfgForm.FormCreate *) ;


procedure TSerwerCfgForm.ExitButtonClick ( Sender : TObject ) ;
begin (* TSerwerCfgForm.ExitButtonClick *)
  Application . Terminate ;
end (* TSerwerCfgForm.ExitButtonClick *) ;


procedure TSerwerCfgForm.OpenPortButtonClick ( Sender : TObject ) ;
var
  OpenFail             : boolean ;
begin (* TSerwerCfgForm.OpenPortButtonClick *)
  OpenFail := false ;
  SerialChannel . Device := SerialPortComboBox . Text ;
  try
    SerialChannel . Open ;
  except
    ShowMessage ( string ( 'Port komunikacyjny ' ) + SerialPortComboBox . Text +
                  string ( ' nie jest dostępny.'  ) ) ;
    OpenFail := true ;
  end (* try *) ;
  if OpenFail then
  begin (* 1 *)
    SerialChannel . Close ;
    exit ;
  end (* 1 *) ;
  SerialChannel . SetDTR ( true ) ;
  SerialChannel . SetRTS ( false ) ;
  TextMemo . Clear ;
  OpenPortButton . Enabled := false ;
  GeneralTimer. Enabled := true ;
  ConnectLED . Color := clGreen ;
  ConnectLED . Active := true ;
  GeneralTimer. Enabled := true ;
  EnableTotalButtons ;
  CyclicBuffer . FrIndex := 0 ;
  CyclicBuffer . ToIndex := 0 ;
  RecvCommand := string ( '' ) ;
  Sleep ( 100 ) ;
  SerialChannel . SetDTR ( false ) ;
end (* TSerwerCfgForm.OpenPortButtonClick *) ;


procedure TSerwerCfgForm.ClosePortButtonClick ( Sender : TObject ) ;
begin (* TSerwerCfgForm.ClosePortButtonClick *)
  ConnectLED . Active := false ;
  GeneralTimer . Enabled := false ;
  SerialChannel . Close ;
  DisableTotalButtons ;
end (* TSerwerCfgForm.ClosePortButtonClick *) ;


procedure TSerwerCfgForm.ResetButtonClick ( Sender : TObject ) ;
begin (* TSerwerCfgForm.ResetButtonClick *)
  SerialChannel . SetDTR ( true ) ;
  Sleep ( 100 ) ;
  SerialChannel . SetDTR ( false ) ;
end (* TSerwerCfgForm.ResetButtonClick *) ;


procedure TSerwerCfgForm.SendButtonClick ( Sender : TObject ) ;
var
  CommandToSend          : string ;
begin (* TSerwerCfgForm.SendButtonClick *)
  CommandToSend := CfgDataEdit . Text + string ( chr ( $0D ) ) ;
  SerialChannel . WriteData ( CommandToSend . UpperCase ( CommandToSend ) ) ;
end (* TSerwerCfgForm.SendButtonClick *) ;


procedure TSerwerCfgForm.EnableTotalButtons ;
begin (* TSerwerCfgForm.EnableTotalButtons *)
  OpenPortButton . Enabled := false ;
  ClosePortButton . Enabled := true ;
  ExitButton . Enabled := false ;
  SendButton . Enabled := true ;
  ResetButton . Enabled := true ;
end (* TSerwerCfgForm.EnableTotalButtons *) ;


procedure TSerwerCfgForm.DisableTotalButtons ;
begin (* TSerwerCfgForm.DisableTotalButtons *)
  OpenPortButton . Enabled := true ;
  ClosePortButton . Enabled := false ;
  ExitButton . Enabled := true ;
  SendButton . Enabled := false ;
  ResetButton . Enabled := false ;
end (* TSerwerCfgForm.DisableTotalButtons *) ;


procedure TSerwerCfgForm.AddToCyclic ( Ch : char ) ;
begin (* TSerwerCfgForm.AddToCyclic *)
  CyclicBuffer . Buffer [ CyclicBuffer . ToIndex ] := Ch ;
  CyclicBuffer . ToIndex := CyclicBuffer . ToIndex + 1 ;
  if CyclicBuffer . ToIndex >= CyclicBufferSize then
    CyclicBuffer . ToIndex := 0 ;
end (* TSerwerCfgForm.AddToCyclic *) ;


function TSerwerCfgForm.GetCh ( ) : char ;
var
  Ch                      : char ;
begin (* TSerwerCfgForm.GetCh *)
  if CyclicBuffer . FrIndex = CyclicBuffer . ToIndex then
  begin (* 1 *)
    Ch := chr ( 0 ) ;
  end (* 1 *)
  else
  begin (* 1 *)
    Ch := CyclicBuffer . Buffer [ CyclicBuffer . FrIndex ] ;
    CyclicBuffer . FrIndex := CyclicBuffer . FrIndex + 1 ;
    if CyclicBuffer . FrIndex >= CyclicBufferSize then
      CyclicBuffer . FrIndex := 0 ;
  end (* 1 *) ;
  Result := Ch ;
end (* TSerwerCfgForm.GetCh *) ;


procedure TSerwerCfgForm.GeneralTimerTimer ( Sender : TObject ) ;
var
  Command                : string ;
  Loop                   : word ;
  StringSize             : word ;
  Ch                     : char ;
begin (* TSerwerCfgForm.GeneralTimerTimer *)
  GeneralTimer . Enabled := false ;
  if SerialChannel . DataAvailable then
  begin (* 1 *)
    Command := SerialChannel . ReadData ;
    StringSize := Command . Length ;
    for Loop := 1 to StringSize do
    begin (* 2 *)
      Ch := Command [ Loop ] ;
      if Ch < ' ' then
      begin (* 3 *)
        if Ch = chr ( $0D ) then
          AddToCyclic ( Ch ) ;
      end (* 3 *)
      else
      begin (* 3 *)
        AddToCyclic ( Ch ) ;
      end (* 3 *) ;
    end (* 2 *) ;
  end (* 1 *) ;
  while CyclicBuffer . FrIndex <> CyclicBuffer . ToIndex do
  begin (* 1 *)
    Ch := GetCh ( ) ;
    if Ch = chr ( $0D ) then
    begin (* 2 *)
      TextMemo . Lines . Add ( RecvCommand ) ;
      RecvCommand := string ( '' ) ;
    end (* 2 *)
    else
    begin (* 2 *)
      RecvCommand := RecvCommand + Ch ;
    end (* 2 *) ;
  end (* 1 *) ;
  GeneralTimer . Enabled := true ;
end (* TSerwerCfgForm.GeneralTimerTimer *) ;

end.

